@@ -25,8 +25,12 @@
  *****************************************************************************/
 #ifdef ANDROID
 #define LOG_TAG "libmei"
-#include <cutils/log.h>
-#define mei_msg(_me, fmt, ARGS...) ALOGV_IF((_me->log_level >= MEI_LOG_LEVEL_VERBOSE), fmt, ##ARGS)
+#include <android/log_macros.h>
+#define mei_msg(_me, fmt, ARGS...) \
+((_me->log_level >= MEI_LOG_LEVEL_VERBOSE) \
+? (void)ALOGV(fmt, ##ARGS) \
+: (void)0)
+
 #define mei_err(_me, fmt, ARGS...) ALOGE(fmt, ##ARGS)
 #ifdef DEBUG
 static inline void __dump_buffer(const char *buf)
@@ -394,13 +398,14 @@ static inline int __mei_getkind(struct mei *me, const char *device, char *kind,
 	}
 	*kind_size = (size_t)len;
 	memcpy(kind, buf, (size_t)len);
+	kind[len - 1] = '\0';
 
 	return 0;
 #undef KIND_FILENAME_LEN
 #undef KIND_LEN
 }
 
-static int mei_init_with_log_int(struct mei *me, const char *device, const uuid_le *guid,
+static int __mei_init_with_log_int(struct mei *me, const char *device, const uuid_le *guid,
 		      unsigned char req_protocol_version, bool verbose,
 		      mei_log_callback log_callback, mei_log_callback2 log_callback2)
 {
@@ -425,13 +430,18 @@ static int mei_init_with_log_int(struct mei *me, const char *device, const uuid_
 
 	rc = __mei_open(me, device);
 	if (rc < 0) {
-		mei_err(me, "Cannot establish a handle to the Intel MEI driver %.20s [%d]:%s\n",
+		if (rc != -ENODEV) {
+			mei_err(me, "Cannot establish a handle to the Intel MEI driver %.20s [%d]:%s\n",
+				device, rc, strerror(-rc));
+			return rc;
+		}
+		mei_msg(me, "Intel MEI driver %.20s returned [%d]:%s\n",
 			device, rc, strerror(-rc));
-		return rc;
+		me->state = MEI_CL_STATE_DISABLED;
+	} else {
+		mei_msg(me, "Opened %.20s: fd = %d\n", device, me->fd);
+		me->state = MEI_CL_STATE_INITIALIZED;
 	}
-
-	mei_msg(me, "Opened %.20s: fd = %d\n", device, me->fd);
-
 	memcpy(&me->guid, guid, sizeof(*guid));
 	me->prot_ver = req_protocol_version;
 	me->device = strdup(device);
@@ -440,8 +450,6 @@ static int mei_init_with_log_int(struct mei *me, const char *device, const uuid_
 		return -ENOMEM;
 	}
 
-	me->state = MEI_CL_STATE_INITIALIZED;
-
 	return 0;
 }
 
@@ -449,7 +457,7 @@ int mei_init_with_log(struct mei *me, const char *device, const uuid_le *guid,
 		      unsigned char req_protocol_version, bool verbose,
 		      mei_log_callback log_callback) 
 {
-	return mei_init_with_log_int(me, device, guid,
+	return __mei_init_with_log_int(me, device, guid,
 		req_protocol_version, verbose, log_callback, NULL);
 }
 
@@ -457,7 +465,7 @@ int mei_init_with_log2(struct mei *me, const char *device, const uuid_le *guid,
 		      unsigned char req_protocol_version, bool verbose,
 		      mei_log_callback2 log_callback) 
 {
-	return mei_init_with_log_int(me, device, guid,
+	return __mei_init_with_log_int(me, device, guid,
 		req_protocol_version, verbose, NULL, log_callback);
 }			  
 
@@ -610,6 +618,11 @@ static int __int_mei_connect(struct mei *me, uint8_t vtag)
 		return -EINVAL;
 	}
 
+	if (me->state == MEI_CL_STATE_DISABLED) {
+		mei_err(me, "client is disabled\n");
+		return -EINVAL;
+	}
+
 	me->vtag = vtag;
 	if (me->vtag) {
 		memset(&data_v, 0, sizeof(data_v));
@@ -665,6 +678,11 @@ ssize_t mei_recv_msg(struct mei *me, unsigned char *buffer, size_t len)
 	if (!me || !buffer)
 		return -EINVAL;
 
+	if (me->state != MEI_CL_STATE_CONNECTED) {
+		mei_err(me, "client is not connected [%d]\n", me->state);
+		return -EINVAL;
+	}
+
 	mei_msg(me, "call read length = %zu\n", len);
 
 	rc = __mei_read(me, buffer, len);
@@ -686,6 +704,11 @@ ssize_t mei_send_msg(struct mei *me, const unsigned char *buffer, size_t len)
 	if (!me || !buffer)
 		return -EINVAL;
 
+	if (me->state != MEI_CL_STATE_CONNECTED) {
+		mei_err(me, "client is not connected [%d]\n", me->state);
+		return -EINVAL;
+	}
+
 	mei_msg(me, "call write length = %zu\n", len);
 	mei_dump_hex_buffer(me, buffer, len);
 
